@@ -84,8 +84,12 @@ static void         ifactory_object_read_done( NAIFactoryObject *instance, const
 static guint        ifactory_object_write_start( NAIFactoryObject *instance, const NAIFactoryProvider *writer, void *writer_data, GSList **messages );
 static guint        ifactory_object_write_done( NAIFactoryObject *instance, const NAIFactoryProvider *writer, void *writer_data, GSList **messages );
 
-static void         convert_pre_v2_action( NAIFactoryObject *instance );
+static void         icontext_iface_init( NAIContextInterface *iface );
+static gboolean     icontext_is_candidate( NAIContext *object, guint target, GList *selection );
+
+static void         convert_v1_to_v2( NAIFactoryObject *instance );
 static void         deals_with_toolbar_label( NAIFactoryObject *instance );
+static void         deals_with_version( NAIFactoryObject *instance );
 
 static gboolean     object_object_is_valid( const NAObjectAction *action );
 static gboolean     is_valid_label( const NAObjectAction *action );
@@ -122,6 +126,12 @@ register_type( void )
 		( GInstanceInitFunc ) instance_init
 	};
 
+	static const GInterfaceInfo icontext_iface_info = {
+		( GInterfaceInitFunc ) icontext_iface_init,
+		NULL,
+		NULL
+	};
+
 	static const GInterfaceInfo ifactory_object_iface_info = {
 		( GInterfaceInitFunc ) ifactory_object_iface_init,
 		NULL,
@@ -132,6 +142,8 @@ register_type( void )
 
 	type = g_type_register_static( NA_OBJECT_ITEM_TYPE, "NAObjectAction", &info, 0 );
 
+	g_type_add_interface_static( type, NA_ICONTEXT_TYPE, &icontext_iface_info );
+
 	g_type_add_interface_static( type, NA_IFACTORY_OBJECT_TYPE, &ifactory_object_iface_info );
 
 	return( type );
@@ -171,13 +183,13 @@ instance_init( GTypeInstance *instance, gpointer klass )
 	static const gchar *thisfn = "na_object_action_instance_init";
 	NAObjectAction *self;
 
-	g_debug( "%s: instance=%p (%s), klass=%p",
-			thisfn, ( void * ) instance, G_OBJECT_TYPE_NAME( instance ), ( void * ) klass );
-
 	g_return_if_fail( NA_IS_OBJECT_ACTION( instance ));
 
 	self = NA_OBJECT_ACTION( instance );
 
+	g_debug( "%s: instance=%p (%s), klass=%p",
+			thisfn, ( void * ) instance, G_OBJECT_TYPE_NAME( instance ), ( void * ) klass );
+
 	self->private = g_new0( NAObjectActionPrivate, 1 );
 }
 
@@ -211,14 +223,14 @@ instance_dispose( GObject *object )
 	static const gchar *thisfn = "na_object_action_instance_dispose";
 	NAObjectAction *self;
 
-	g_debug( "%s: object=%p (%s)", thisfn, ( void * ) object, G_OBJECT_TYPE_NAME( object ));
-
 	g_return_if_fail( NA_IS_OBJECT_ACTION( object ));
 
 	self = NA_OBJECT_ACTION( object );
 
 	if( !self->private->dispose_has_run ){
 
+		g_debug( "%s: object=%p (%s)", thisfn, ( void * ) object, G_OBJECT_TYPE_NAME( object ));
+
 		self->private->dispose_has_run = TRUE;
 
 		/* chain up to the parent class */
@@ -234,12 +246,12 @@ instance_finalize( GObject *object )
 	static const gchar *thisfn = "na_object_action_instance_finalize";
 	NAObjectAction *self;
 
-	g_debug( "%s: object=%p (%s)", thisfn, ( void * ) object, G_OBJECT_TYPE_NAME( object ));
-
 	g_return_if_fail( NA_IS_OBJECT_ACTION( object ));
 
 	self = NA_OBJECT_ACTION( object );
 
+	g_debug( "%s: object=%p (%s)", thisfn, ( void * ) object, G_OBJECT_TYPE_NAME( object ));
+
 	g_free( self->private );
 
 	/* chain call to parent class */
@@ -330,12 +342,19 @@ ifactory_object_read_done( NAIFactoryObject *instance, const NAIFactoryProvider
 
 	/* may attach a new profile if we detect a pre-v2 action
 	 */
-	convert_pre_v2_action( instance );
+	convert_v1_to_v2( instance );
 
 	/* deals with obsoleted data, i.e. data which may have been written in the past
 	 * but are no long written by now
 	 */
 	deals_with_toolbar_label( instance );
+	deals_with_version( instance );
+
+	/* prepare the context after the reading
+	 */
+	na_icontext_read_done( NA_ICONTEXT( instance ));
+
+	na_object_dump( instance );
 
 	/* last, set other action defaults
 	 */
@@ -345,7 +364,7 @@ ifactory_object_read_done( NAIFactoryObject *instance, const NAIFactoryProvider
 static guint
 ifactory_object_write_start( NAIFactoryObject *instance, const NAIFactoryProvider *writer, void *writer_data, GSList **messages )
 {
-	na_object_item_factory_write_start( NA_OBJECT_ITEM( instance ));
+	na_object_item_rebuild_children_slist( NA_OBJECT_ITEM( instance ));
 
 	return( NA_IIO_PROVIDER_CODE_OK );
 }
@@ -378,15 +397,32 @@ ifactory_object_write_done( NAIFactoryObject *instance, const NAIFactoryProvider
 	return( code );
 }
 
+static void
+icontext_iface_init( NAIContextInterface *iface )
+{
+	static const gchar *thisfn = "na_object_action_icontext_iface_init";
+
+	g_debug( "%s: iface=%p", thisfn, ( void * ) iface );
+
+	iface->is_candidate = icontext_is_candidate;
+}
+
+static gboolean
+icontext_is_candidate( NAIContext *object, guint target, GList *selection )
+{
+	return( TRUE );
+}
+
 /*
  * do we have a pre-v2 action ?
- *  it may be identified by an version = "1.x"
+ *  it is be identified by an version = "1.x"
  *  or by any data found in data_def_action_v1 (defined in na-object-action-factory.c)
- *  -> move obsoleted data to a new profile
+ *  -> move obsoleted data to a new profile, updating the version string
  */
 static void
-convert_pre_v2_action( NAIFactoryObject *instance )
+convert_v1_to_v2( NAIFactoryObject *instance )
 {
+	static const gchar *thisfn = "na_object_action_convert_v1_to_v2";
 	gboolean is_pre_v2;
 	GList *to_move;
 	NADataDef *def;
@@ -402,10 +438,9 @@ convert_pre_v2_action( NAIFactoryObject *instance )
 	while( def->name ){
 		boxed = na_ifactory_object_get_data_boxed( instance , def->name );
 		if( boxed ){
-			g_debug( "na_object_action_convert_pre_v2_action: " \
-					 "boxed=%p (%s) marked to be moved from action body to profile",
-							 ( void * ) boxed, def->name );
-			to_move =g_list_prepend( to_move, boxed );
+			g_debug( "%s: boxed=%p (%s) marked to be moved from action body to profile",
+							 thisfn, ( void * ) boxed, def->name );
+			to_move = g_list_prepend( to_move, boxed );
 		}
 		def++;
 	}
@@ -435,6 +470,7 @@ convert_pre_v2_action( NAIFactoryObject *instance )
 		}
 
 		na_factory_object_set_defaults( NA_IFACTORY_OBJECT( profile ));
+		na_object_set_version( instance, "2.0" );
 	}
 }
 
@@ -464,6 +500,36 @@ deals_with_toolbar_label( NAIFactoryObject *instance )
 	g_free( toolbar_label );
 }
 
+/*
+ * manage the conversion from version string to version number
+ * -> pre-v2 actions have already been converted to a version string "2.0"
+ * -> if we have here no version string nor version number, then we may
+ *    consider that this is an item written by the 2.30 N-A version
+ *    so we just force a version number to 2
+ */
+static void
+deals_with_version( NAIFactoryObject *instance )
+{
+	guint version_uint;
+	gchar *version_str;
+
+	version_uint = na_object_get_iversion( instance );
+
+	if( !version_uint ){
+		version_str = na_object_get_version( instance );
+
+		if( !version_str || !strlen( version_str )){
+			g_free( version_str );
+			version_str = g_strdup( "2.0" );
+		}
+
+		version_uint = atoi( version_str );
+		na_object_set_iversion( instance, version_uint );
+
+		g_free( version_str );
+	}
+}
+
 static gboolean
 object_object_is_valid( const NAObjectAction *action )
 {
@@ -679,6 +745,23 @@ na_object_action_attach_profile( NAObjectAction *action, NAObjectProfile *profil
 	}
 }
 
+/**
+ * na_object_action_set_last_version:
+ * @action: the #NAObjectAction action to update.
+ *
+ * Set the version number of the @action to the last one.
+ */
+void
+na_object_action_set_last_version( NAObjectAction *action )
+{
+	g_return_if_fail( NA_IS_OBJECT_ACTION( action ));
+
+	if( !action->private->dispose_has_run ){
+
+		na_object_set_version( action, "2.0" );
+	}
+}
+
 /**
  * na_object_action_is_candidate:
  * @action: the #NAObjectAction to be tested.
