@@ -49,9 +49,14 @@
 #include "nact-iactions-list.h"
 #include "nact-iaction-tab.h"
 #include "nact-icommand-tab.h"
+#include "nact-ibasenames-tab.h"
+#include "nact-imimetypes-tab.h"
 #include "nact-ifolders-tab.h"
-#include "nact-iconditions-tab.h"
-#include "nact-iadvanced-tab.h"
+#include "nact-ischemes-tab.h"
+#include "nact-icapabilities-tab.h"
+#include "nact-ienvironment-tab.h"
+#include "nact-iexecution-tab.h"
+#include "nact-iproperties-tab.h"
 #include "nact-main-tab.h"
 #include "nact-main-menubar.h"
 #include "nact-main-statusbar.h"
@@ -75,38 +80,33 @@ struct NactMainWindowPrivate {
 	GList           *deleted;
 
 	/**
-	 * Currently edited action or menu.
+	 * Currently selected action or menu.
 	 *
-	 * This is the action or menu which is displayed in tab Action ;
-	 * it may be different of the row being currently selected.
+	 * This is the action or menu which is displayed in tabs Action/Menu
+	 * and Properties ; it may be different of the exact row being currently
+	 * selected, e.g. when a sub-profile is edited.
 	 *
-	 * Can be null, and this implies that edited_profile is also null.
+	 * Can be null, and this implies that @selected_profile is also null,
+	 * e.g. when the list is empty or in case of multiple selection.
 	 *
-	 * 'editable_item' property is computed on selection change;
-	 * This is the real writability status of the item.
+	 * 'editable' property is computed on selection change;
+	 * This is the real writability status of the item at this time.
 	 */
-	NAObjectItem    *edited_item;
+	NAObjectItem    *selected_item;
 	gboolean         editable;
 	gint             reason;
 
 	/**
-	 * Currently edited profile.
+	 * Currently selected profile.
 	 *
-	 * This is the profile which is displayed in tabs Command,
-	 * Conditions and Advanced ; it may be different of the row being
-	 * currently selected.
+	 * This is the profile which is displayed in tab Command;
+	 * it may be different of the exact row being currently selected.
 	 *
-	 * Can be null if @edited_item is a menu, or an action with more
+	 * Can be null if @selected_item is a menu, or an action with more
 	 * than one profile and action is selected, or an action without
-	 * any profile.
+	 * any profile, or the list is empty, or in case of multiple selection.
 	 */
-	NAObjectProfile *edited_profile;
-
-	/**
-	 * Currently selected row.
-	 * May be null if list is empty or selection is multiple.
-	 */
-	NAObjectId      *selected_row;
+	NAObjectProfile *selected_profile;
 
 	/**
 	 * The convenience clipboard object.
@@ -120,7 +120,6 @@ struct NactMainWindowPrivate {
 enum {
 	PROP_EDITED_ITEM = 1,
 	PROP_EDITED_PROFILE,
-	PROP_SELECTED_ROW,
 	PROP_EDITABLE,
 	PROP_REASON
 };
@@ -131,7 +130,6 @@ enum {
 	PROVIDER_CHANGED,
 	SELECTION_CHANGED,
 	ITEM_UPDATED,
-	ENABLE_TAB,
 	UPDATE_SENSITIVITIES,
 	ORDER_CHANGED,
 	LAST_SIGNAL
@@ -145,9 +143,14 @@ static void     class_init( NactMainWindowClass *klass );
 static void     iactions_list_iface_init( NactIActionsListInterface *iface );
 static void     iaction_tab_iface_init( NactIActionTabInterface *iface );
 static void     icommand_tab_iface_init( NactICommandTabInterface *iface );
+static void     ibasenames_tab_iface_init( NactIBasenamesTabInterface *iface );
+static void     imimetypes_tab_iface_init( NactIMimetypesTabInterface *iface );
 static void     ifolders_tab_iface_init( NactIFoldersTabInterface *iface );
-static void     iconditions_tab_iface_init( NactIConditionsTabInterface *iface );
-static void     iadvanced_tab_iface_init( NactIAdvancedTabInterface *iface );
+static void     ischemes_tab_iface_init( NactISchemesTabInterface *iface );
+static void     icapabilities_tab_iface_init( NactICapabilitiesTabInterface *iface );
+static void     ienvironment_tab_iface_init( NactIEnvironmentTabInterface *iface );
+static void     iexecution_tab_iface_init( NactIExecutionTabInterface *iface );
+static void     iproperties_tab_iface_init( NactIPropertiesTabInterface *iface );
 static void     iabout_iface_init( NAIAboutInterface *iface );
 static void     ipivot_consumer_iface_init( NAIPivotConsumerInterface *iface );
 static void     iprefs_base_iface_init( BaseIPrefsInterface *iface );
@@ -169,6 +172,7 @@ static void     on_base_all_widgets_showed( NactMainWindow *window, gpointer use
 static void     on_main_window_level_zero_order_changed( NactMainWindow *window, gpointer user_data );
 static void     on_iactions_list_selection_changed( NactIActionsList *instance, GSList *selected_items );
 static void     on_iactions_list_status_changed( NactMainWindow *window, gpointer user_data );
+static void     raz_main_properties( NactMainWindow *window );
 static void     set_current_object_item( NactMainWindow *window, GSList *selected_items );
 static void     set_current_profile( NactMainWindow *window, gboolean set_action, GSList *selected_items );
 static gchar   *iactions_list_get_treeview_name( NactIActionsList *instance );
@@ -233,20 +237,50 @@ register_type( void )
 		NULL
 	};
 
+	static const GInterfaceInfo ibasenames_tab_iface_info = {
+		( GInterfaceInitFunc ) ibasenames_tab_iface_init,
+		NULL,
+		NULL
+	};
+
+	static const GInterfaceInfo imimetypes_tab_iface_info = {
+		( GInterfaceInitFunc ) imimetypes_tab_iface_init,
+		NULL,
+		NULL
+	};
+
 	static const GInterfaceInfo ifolders_tab_iface_info = {
 		( GInterfaceInitFunc ) ifolders_tab_iface_init,
 		NULL,
 		NULL
 	};
 
-	static const GInterfaceInfo iconditions_tab_iface_info = {
-		( GInterfaceInitFunc ) iconditions_tab_iface_init,
+	static const GInterfaceInfo ischemes_tab_iface_info = {
+		( GInterfaceInitFunc ) ischemes_tab_iface_init,
 		NULL,
 		NULL
 	};
 
-	static const GInterfaceInfo iadvanced_tab_iface_info = {
-		( GInterfaceInitFunc ) iadvanced_tab_iface_init,
+	static const GInterfaceInfo icapabilities_tab_iface_info = {
+		( GInterfaceInitFunc ) icapabilities_tab_iface_init,
+		NULL,
+		NULL
+	};
+
+	static const GInterfaceInfo ienvironment_tab_iface_info = {
+		( GInterfaceInitFunc ) ienvironment_tab_iface_init,
+		NULL,
+		NULL
+	};
+
+	static const GInterfaceInfo iexecution_tab_iface_info = {
+		( GInterfaceInitFunc ) iexecution_tab_iface_init,
+		NULL,
+		NULL
+	};
+
+	static const GInterfaceInfo iproperties_tab_iface_info = {
+		( GInterfaceInitFunc ) iproperties_tab_iface_init,
 		NULL,
 		NULL
 	};
@@ -279,11 +313,21 @@ register_type( void )
 
 	g_type_add_interface_static( type, NACT_ICOMMAND_TAB_TYPE, &icommand_tab_iface_info );
 
+	g_type_add_interface_static( type, NACT_IBASENAMES_TAB_TYPE, &ibasenames_tab_iface_info );
+
+	g_type_add_interface_static( type, NACT_IMIMETYPES_TAB_TYPE, &imimetypes_tab_iface_info );
+
 	g_type_add_interface_static( type, NACT_IFOLDERS_TAB_TYPE, &ifolders_tab_iface_info );
 
-	g_type_add_interface_static( type, NACT_ICONDITIONS_TAB_TYPE, &iconditions_tab_iface_info );
+	g_type_add_interface_static( type, NACT_ISCHEMES_TAB_TYPE, &ischemes_tab_iface_info );
 
-	g_type_add_interface_static( type, NACT_IADVANCED_TAB_TYPE, &iadvanced_tab_iface_info );
+	g_type_add_interface_static( type, NACT_ICAPABILITIES_TAB_TYPE, &icapabilities_tab_iface_info );
+
+	g_type_add_interface_static( type, NACT_IENVIRONMENT_TAB_TYPE, &ienvironment_tab_iface_info );
+
+	g_type_add_interface_static( type, NACT_IEXECUTION_TAB_TYPE, &iexecution_tab_iface_info );
+
+	g_type_add_interface_static( type, NACT_IPROPERTIES_TAB_TYPE, &iproperties_tab_iface_info );
 
 	g_type_add_interface_static( type, NA_IABOUT_TYPE, &iabout_iface_info );
 
@@ -313,26 +357,19 @@ class_init( NactMainWindowClass *klass )
 	object_class->get_property = instance_get_property;
 
 	spec = g_param_spec_pointer(
-			TAB_UPDATABLE_PROP_EDITED_ACTION,
+			TAB_UPDATABLE_PROP_SELECTED_ITEM,
 			"Edited NAObjectItem",
 			"A pointer to the edited NAObjectItem, an action or a menu",
 			G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE );
 	g_object_class_install_property( object_class, PROP_EDITED_ITEM, spec );
 
 	spec = g_param_spec_pointer(
-			TAB_UPDATABLE_PROP_EDITED_PROFILE,
+			TAB_UPDATABLE_PROP_SELECTED_PROFILE,
 			"Edited NAObjectProfile",
 			"A pointer to the edited NAObjectProfile",
 			G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE );
 	g_object_class_install_property( object_class, PROP_EDITED_PROFILE, spec );
 
-	spec = g_param_spec_pointer(
-			TAB_UPDATABLE_PROP_SELECTED_ROW,
-			"Selected NAObjectId",
-			"A pointer to the currently selected row",
-			G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE );
-	g_object_class_install_property( object_class, PROP_SELECTED_ROW, spec );
-
 	spec = g_param_spec_boolean(
 			TAB_UPDATABLE_PROP_EDITABLE,
 			"Editable item ?",
@@ -424,25 +461,6 @@ class_init( NactMainWindowClass *klass )
 			G_TYPE_POINTER,
 			G_TYPE_BOOLEAN );
 
-	/**
-	 * nact-tab-updatable-enable-tab:
-	 *
-	 * This signal is emitted by the IActionTab when the nature of the
-	 * item has been modified: some tabs should probably be enabled or
-	 * disabled
-	 */
-	st_signals[ ENABLE_TAB ] = g_signal_new(
-			TAB_UPDATABLE_SIGNAL_ENABLE_TAB,
-			G_TYPE_OBJECT,
-			G_SIGNAL_RUN_LAST,
-			0,					/* no default handler */
-			NULL,
-			NULL,
-			g_cclosure_marshal_VOID__POINTER,
-			G_TYPE_NONE,
-			1,
-			G_TYPE_POINTER );
-
 	/**
 	 * main-window-update-sensitivities:
 	 *
@@ -506,6 +524,22 @@ icommand_tab_iface_init( NactICommandTabInterface *iface )
 	g_debug( "%s: iface=%p", thisfn, ( void * ) iface );
 }
 
+static void
+ibasenames_tab_iface_init( NactIBasenamesTabInterface *iface )
+{
+	static const gchar *thisfn = "nact_main_window_ibasenames_tab_iface_init";
+
+	g_debug( "%s: iface=%p", thisfn, ( void * ) iface );
+}
+
+static void
+imimetypes_tab_iface_init( NactIMimetypesTabInterface *iface )
+{
+	static const gchar *thisfn = "nact_main_window_imimetypes_tab_iface_init";
+
+	g_debug( "%s: iface=%p", thisfn, ( void * ) iface );
+}
+
 static void
 ifolders_tab_iface_init( NactIFoldersTabInterface *iface )
 {
@@ -515,17 +549,41 @@ ifolders_tab_iface_init( NactIFoldersTabInterface *iface )
 }
 
 static void
-iconditions_tab_iface_init( NactIConditionsTabInterface *iface )
+ischemes_tab_iface_init( NactISchemesTabInterface *iface )
 {
-	static const gchar *thisfn = "nact_main_window_iconditions_tab_iface_init";
+	static const gchar *thisfn = "nact_main_window_ischemes_tab_iface_init";
 
 	g_debug( "%s: iface=%p", thisfn, ( void * ) iface );
 }
 
 static void
-iadvanced_tab_iface_init( NactIAdvancedTabInterface *iface )
+icapabilities_tab_iface_init( NactICapabilitiesTabInterface *iface )
 {
-	static const gchar *thisfn = "nact_main_window_iadvanced_tab_iface_init";
+	static const gchar *thisfn = "nact_main_window_icapabilities_tab_iface_init";
+
+	g_debug( "%s: iface=%p", thisfn, ( void * ) iface );
+}
+
+static void
+ienvironment_tab_iface_init( NactIEnvironmentTabInterface *iface )
+{
+	static const gchar *thisfn = "nact_main_window_ienvironment_tab_iface_init";
+
+	g_debug( "%s: iface=%p", thisfn, ( void * ) iface );
+}
+
+static void
+iexecution_tab_iface_init( NactIExecutionTabInterface *iface )
+{
+	static const gchar *thisfn = "nact_main_window_iexecution_tab_iface_init";
+
+	g_debug( "%s: iface=%p", thisfn, ( void * ) iface );
+}
+
+static void
+iproperties_tab_iface_init( NactIPropertiesTabInterface *iface )
+{
+	static const gchar *thisfn = "nact_main_window_iproperties_tab_iface_init";
 
 	g_debug( "%s: iface=%p", thisfn, ( void * ) iface );
 }
@@ -614,15 +672,11 @@ instance_get_property( GObject *object, guint property_id, GValue *value, GParam
 
 		switch( property_id ){
 			case PROP_EDITED_ITEM:
-				g_value_set_pointer( value, self->private->edited_item );
+				g_value_set_pointer( value, self->private->selected_item );
 				break;
 
 			case PROP_EDITED_PROFILE:
-				g_value_set_pointer( value, self->private->edited_profile );
-				break;
-
-			case PROP_SELECTED_ROW:
-				g_value_set_pointer( value, self->private->selected_row );
+				g_value_set_pointer( value, self->private->selected_profile );
 				break;
 
 			case PROP_EDITABLE:
@@ -652,15 +706,11 @@ instance_set_property( GObject *object, guint property_id, const GValue *value,
 
 		switch( property_id ){
 			case PROP_EDITED_ITEM:
-				self->private->edited_item = g_value_get_pointer( value );
+				self->private->selected_item = g_value_get_pointer( value );
 				break;
 
 			case PROP_EDITED_PROFILE:
-				self->private->edited_profile = g_value_get_pointer( value );
-				break;
-
-			case PROP_SELECTED_ROW:
-				self->private->selected_row = g_value_get_pointer( value );
+				self->private->selected_profile = g_value_get_pointer( value );
 				break;
 
 			case PROP_EDITABLE:
@@ -710,9 +760,14 @@ instance_dispose( GObject *window )
 		nact_sort_buttons_dispose( self );
 		nact_iaction_tab_dispose( NACT_IACTION_TAB( window ));
 		nact_icommand_tab_dispose( NACT_ICOMMAND_TAB( window ));
+		nact_ibasenames_tab_dispose( NACT_IBASENAMES_TAB( window ));
+		nact_imimetypes_tab_dispose( NACT_IMIMETYPES_TAB( window ));
 		nact_ifolders_tab_dispose( NACT_IFOLDERS_TAB( window ));
-		nact_iconditions_tab_dispose( NACT_ICONDITIONS_TAB( window ));
-		nact_iadvanced_tab_dispose( NACT_IADVANCED_TAB( window ));
+		nact_ischemes_tab_dispose( NACT_ISCHEMES_TAB( window ));
+		nact_icapabilities_tab_dispose( NACT_ICAPABILITIES_TAB( window ));
+		nact_ienvironment_tab_dispose( NACT_IENVIRONMENT_TAB( window ));
+		nact_iexecution_tab_dispose( NACT_IEXECUTION_TAB( window ));
+		nact_iproperties_tab_dispose( NACT_IPROPERTIES_TAB( window ));
 		nact_main_menubar_dispose( self );
 
 		/* chain up to the parent class */
@@ -971,6 +1026,7 @@ actually_delete_item( NactMainWindow *window, NAObject *item, NAUpdater *updater
 		origin = ( NAObject * ) na_object_get_origin( item );
 		if( origin ){
 			na_updater_remove_item( updater, origin );
+			g_object_unref( origin );
 		}
 	}
 }
@@ -1033,9 +1089,14 @@ on_base_initial_load_toplevel( NactMainWindow *window, gpointer user_data )
 
 		nact_iaction_tab_initial_load_toplevel( NACT_IACTION_TAB( window ));
 		nact_icommand_tab_initial_load_toplevel( NACT_ICOMMAND_TAB( window ));
+		nact_ibasenames_tab_initial_load_toplevel( NACT_IBASENAMES_TAB( window ));
+		nact_imimetypes_tab_initial_load_toplevel( NACT_IMIMETYPES_TAB( window ));
 		nact_ifolders_tab_initial_load_toplevel( NACT_IFOLDERS_TAB( window ));
-		nact_iconditions_tab_initial_load_toplevel( NACT_ICONDITIONS_TAB( window ));
-		nact_iadvanced_tab_initial_load_toplevel( NACT_IADVANCED_TAB( window ));
+		nact_ischemes_tab_initial_load_toplevel( NACT_ISCHEMES_TAB( window ));
+		nact_icapabilities_tab_initial_load_toplevel( NACT_ICAPABILITIES_TAB( window ));
+		nact_ienvironment_tab_initial_load_toplevel( NACT_IENVIRONMENT_TAB( window ));
+		nact_iexecution_tab_initial_load_toplevel( NACT_IEXECUTION_TAB( window ));
+		nact_iproperties_tab_initial_load_toplevel( NACT_IPROPERTIES_TAB( window ));
 
 		nact_main_statusbar_initial_load_toplevel( window );
 	}
@@ -1070,9 +1131,15 @@ on_base_runtime_init_toplevel( NactMainWindow *window, gpointer user_data )
 
 		nact_iaction_tab_runtime_init_toplevel( NACT_IACTION_TAB( window ));
 		nact_icommand_tab_runtime_init_toplevel( NACT_ICOMMAND_TAB( window ));
+		nact_ibasenames_tab_runtime_init_toplevel( NACT_IBASENAMES_TAB( window ));
+		nact_imimetypes_tab_runtime_init_toplevel( NACT_IMIMETYPES_TAB( window ));
 		nact_ifolders_tab_runtime_init_toplevel( NACT_IFOLDERS_TAB( window ));
-		nact_iconditions_tab_runtime_init_toplevel( NACT_ICONDITIONS_TAB( window ));
-		nact_iadvanced_tab_runtime_init_toplevel( NACT_IADVANCED_TAB( window ));
+		nact_ischemes_tab_runtime_init_toplevel( NACT_ISCHEMES_TAB( window ));
+		nact_icapabilities_tab_runtime_init_toplevel( NACT_ICAPABILITIES_TAB( window ));
+		nact_ienvironment_tab_runtime_init_toplevel( NACT_IENVIRONMENT_TAB( window ));
+		nact_iexecution_tab_runtime_init_toplevel( NACT_IEXECUTION_TAB( window ));
+		nact_iproperties_tab_runtime_init_toplevel( NACT_IPROPERTIES_TAB( window ));
+
 		nact_main_menubar_runtime_init( window );
 
 		order_mode = na_iprefs_get_order_mode( NA_IPREFS( updater ));
@@ -1109,17 +1176,28 @@ on_base_all_widgets_showed( NactMainWindow *window, gpointer user_data )
 	g_return_if_fail( NACT_IS_IACTIONS_LIST( window ));
 	g_return_if_fail( NACT_IS_IACTION_TAB( window ));
 	g_return_if_fail( NACT_IS_ICOMMAND_TAB( window ));
+	g_return_if_fail( NACT_IS_IBASENAMES_TAB( window ));
+	g_return_if_fail( NACT_IS_IMIMETYPES_TAB( window ));
 	g_return_if_fail( NACT_IS_IFOLDERS_TAB( window ));
-	g_return_if_fail( NACT_IS_ICONDITIONS_TAB( window ));
-	g_return_if_fail( NACT_IS_IADVANCED_TAB( window ));
+	g_return_if_fail( NACT_IS_ISCHEMES_TAB( window ));
+	g_return_if_fail( NACT_IS_IENVIRONMENT_TAB( window ));
+	g_return_if_fail( NACT_IS_IEXECUTION_TAB( window ));
+	g_return_if_fail( NACT_IS_IPROPERTIES_TAB( window ));
 
 	if( !window->private->dispose_has_run ){
 
+		raz_main_properties( window );
+
 		nact_iaction_tab_all_widgets_showed( NACT_IACTION_TAB( window ));
 		nact_icommand_tab_all_widgets_showed( NACT_ICOMMAND_TAB( window ));
+		nact_ibasenames_tab_all_widgets_showed( NACT_IBASENAMES_TAB( window ));
+		nact_imimetypes_tab_all_widgets_showed( NACT_IMIMETYPES_TAB( window ));
 		nact_ifolders_tab_all_widgets_showed( NACT_IFOLDERS_TAB( window ));
-		nact_iconditions_tab_all_widgets_showed( NACT_ICONDITIONS_TAB( window ));
-		nact_iadvanced_tab_all_widgets_showed( NACT_IADVANCED_TAB( window ));
+		nact_ischemes_tab_all_widgets_showed( NACT_ISCHEMES_TAB( window ));
+		nact_icapabilities_tab_all_widgets_showed( NACT_ICAPABILITIES_TAB( window ));
+		nact_ienvironment_tab_all_widgets_showed( NACT_IENVIRONMENT_TAB( window ));
+		nact_iexecution_tab_all_widgets_showed( NACT_IEXECUTION_TAB( window ));
+		nact_iproperties_tab_all_widgets_showed( NACT_IPROPERTIES_TAB( window ));
 
 		nact_iactions_list_all_widgets_showed( NACT_IACTIONS_LIST( window ));
 		nact_sort_buttons_all_widgets_showed( window );
@@ -1160,30 +1238,25 @@ on_iactions_list_selection_changed( NactIActionsList *instance, GSList *selected
 		return;
 	}
 
-	window->private->selected_row = NULL;
-	window->private->edited_item = NULL;
-	window->private->editable = FALSE;
-	window->private->reason = 0;
-	nact_main_statusbar_set_locked( window, FALSE, 0 );
+	raz_main_properties( window );
 
 	if( count == 1 ){
 		g_return_if_fail( NA_IS_OBJECT_ID( selected_items->data ));
 		object = NA_OBJECT( selected_items->data );
-		window->private->selected_row = NA_OBJECT_ID( object );
 
 		if( NA_IS_OBJECT_ITEM( object )){
-			window->private->edited_item = NA_OBJECT_ITEM( object );
+			window->private->selected_item = NA_OBJECT_ITEM( object );
 			set_current_object_item( window, selected_items );
 
 		} else {
 			g_assert( NA_IS_OBJECT_PROFILE( object ));
-			window->private->edited_profile = NA_OBJECT_PROFILE( object );
+			window->private->selected_profile = NA_OBJECT_PROFILE( object );
 			set_current_profile( window, TRUE, selected_items );
 		}
 
 		application = NACT_APPLICATION( base_window_get_application( BASE_WINDOW( instance )));
 		updater = nact_application_get_updater( application );
-		window->private->editable = na_updater_is_item_writable( updater, window->private->edited_item, &window->private->reason );
+		window->private->editable = na_updater_is_item_writable( updater, window->private->selected_item, &window->private->reason );
 		nact_main_statusbar_set_locked( window, !window->private->editable, window->private->reason );
 
 	} else {
@@ -1195,6 +1268,17 @@ on_iactions_list_selection_changed( NactIActionsList *instance, GSList *selected
 	g_signal_emit_by_name( window, MAIN_WINDOW_SIGNAL_SELECTION_CHANGED, GINT_TO_POINTER( count ));
 }
 
+static void
+raz_main_properties( NactMainWindow *window )
+{
+	window->private->selected_item = NULL;
+	window->private->selected_profile = NULL;
+	window->private->editable = FALSE;
+	window->private->reason = 0;
+
+	nact_main_statusbar_set_locked( window, FALSE, 0 );
+}
+
 static void
 on_iactions_list_status_changed( NactMainWindow *window, gpointer user_data )
 {
@@ -1217,22 +1301,22 @@ set_current_object_item( NactMainWindow *window, GSList *selected_items )
 	/*NAObject *current;*/
 
 	g_debug( "%s: window=%p, current=%p, selected_items=%p",
-			thisfn, ( void * ) window, ( void * ) window->private->edited_item, ( void * ) selected_items );
+			thisfn, ( void * ) window, ( void * ) window->private->selected_item, ( void * ) selected_items );
 
 	/* set the profile to be displayed, if any
 	 */
-	window->private->edited_profile = NULL;
+	window->private->selected_profile = NULL;
 
-	if( window->private->edited_item ){
+	if( window->private->selected_item ){
 
-		if( NA_IS_OBJECT_ACTION( window->private->edited_item )){
+		if( NA_IS_OBJECT_ACTION( window->private->selected_item )){
 
-			count_profiles = na_object_get_items_count( window->private->edited_item );
+			count_profiles = na_object_get_items_count( window->private->selected_item );
 			/*g_return_if_fail( count_profiles >= 1 );*/
 
 			if( count_profiles == 1 ){
-				profiles = na_object_get_items( window->private->edited_item );
-				window->private->edited_profile = NA_OBJECT_PROFILE( profiles->data );
+				profiles = na_object_get_items( window->private->selected_item );
+				window->private->selected_profile = NA_OBJECT_PROFILE( profiles->data );
 			}
 		}
 	}
@@ -1248,13 +1332,13 @@ set_current_profile( NactMainWindow *window, gboolean set_action, GSList *select
 	g_debug( "%s: window=%p, set_action=%s, selected_items=%p",
 			thisfn, ( void * ) window, set_action ? "True":"False", ( void * ) selected_items );
 
-	if( window->private->edited_profile && set_action ){
+	if( window->private->selected_profile && set_action ){
 
-		NAObjectAction *action = NA_OBJECT_ACTION( na_object_get_parent( window->private->edited_profile ));
+		NAObjectAction *action = NA_OBJECT_ACTION( na_object_get_parent( window->private->selected_profile ));
 		NactApplication *application = NACT_APPLICATION( base_window_get_application( BASE_WINDOW( window )));
 		NAUpdater *updater = nact_application_get_updater( application );
-		window->private->edited_item = NA_OBJECT_ITEM( action );
-		window->private->editable = na_updater_is_item_writable( updater, window->private->edited_item, &window->private->reason );
+		window->private->selected_item = NA_OBJECT_ITEM( action );
+		window->private->editable = na_updater_is_item_writable( updater, window->private->selected_item, &window->private->reason );
 	}
 }
 
@@ -1291,8 +1375,8 @@ setup_dialog_title( NactMainWindow *window )
 	application = NACT_APPLICATION( base_window_get_application( BASE_WINDOW( window )));
 	title = base_application_get_application_name( BASE_APPLICATION( application ));
 
-	if( window->private->edited_item ){
-		label = na_object_get_label( window->private->edited_item );
+	if( window->private->selected_item ){
+		label = na_object_get_label( window->private->selected_item );
 		tmp = g_strdup_printf( "%s - %s", title, label );
 		g_free( label );
 		g_free( title );
@@ -1426,9 +1510,8 @@ reload( NactMainWindow *window )
 
 	if( !window->private->dispose_has_run ){
 
-		window->private->edited_item = NULL;
-		window->private->edited_profile = NULL;
-		window->private->selected_row = NULL;
+		window->private->selected_item = NULL;
+		window->private->selected_profile = NULL;
 
 		na_object_unref_items( window->private->deleted );
 		window->private->deleted = NULL;
@@ -1449,17 +1532,26 @@ static void
 ipivot_consumer_on_display_order_changed( NAIPivotConsumer *instance, gint order_mode )
 {
 	static const gchar *thisfn = "nact_main_window_ipivot_consumer_on_display_order_changed";
-	/*NactMainWindow *self;*/
+	NactApplication *application;
+	NAUpdater *updater;
+	GList *tree;
 
+	g_return_if_fail( NACT_IS_MAIN_WINDOW( instance ));
 	g_debug( "%s: instance=%p, order_mode=%d", thisfn, ( void * ) instance, order_mode );
-	g_assert( NACT_IS_MAIN_WINDOW( instance ));
-	/*self = NACT_MAIN_WINDOW( instance );*/
 
 	nact_iactions_list_display_order_change( NACT_IACTIONS_LIST( instance ), order_mode );
 	nact_sort_buttons_display_order_change( NACT_MAIN_WINDOW( instance ), order_mode );
 
-	g_signal_emit_by_name(
-			NACT_MAIN_WINDOW( instance ), MAIN_WINDOW_SIGNAL_LEVEL_ZERO_ORDER_CHANGED, GINT_TO_POINTER( TRUE ));
+	application = NACT_APPLICATION( base_window_get_application( BASE_WINDOW( instance )));
+	updater = nact_application_get_updater( application );
+	tree = na_pivot_get_items( NA_PIVOT( updater ));
+
+	if( g_list_length( tree )){
+		g_signal_emit_by_name(
+				NACT_MAIN_WINDOW( instance ),
+				MAIN_WINDOW_SIGNAL_LEVEL_ZERO_ORDER_CHANGED,
+				GINT_TO_POINTER( TRUE ));
+	}
 }
 
 static void
