@@ -40,6 +40,7 @@
 #include <libnautilus-extension/nautilus-file-info.h>
 #include <libnautilus-extension/nautilus-menu-provider.h>
 
+#include <api/na-core-utils.h>
 #include <api/na-object-api.h>
 
 #include <core/na-pivot.h>
@@ -47,6 +48,7 @@
 #include <core/na-iprefs.h>
 #include <core/na-ipivot-consumer.h>
 #include <core/na-selected-info.h>
+#include <core/na-tokens.h>
 
 #include "nautilus-actions.h"
 
@@ -86,8 +88,10 @@ static GList            *menu_provider_get_background_items( NautilusMenuProvide
 static GList            *menu_provider_get_file_items( NautilusMenuProvider *provider, GtkWidget *window, GList *files );
 static GList            *menu_provider_get_toolbar_items( NautilusMenuProvider *provider, GtkWidget *window, NautilusFileInfo *current_folder );
 
-static GList            *get_file_or_background_items( NautilusActions *plugin, guint target, void *selection );
-static GList            *build_nautilus_menus( NautilusActions *plugin, GList *tree, guint target, GList *files );
+static GList            *get_menus_items( NautilusActions *plugin, guint target, GList *selection );
+static GList            *expand_tokens( GList *tree, NATokens *tokens );
+static NAObjectItem     *expand_tokens_item( NAObjectItem *item, NATokens *tokens );
+static GList            *build_nautilus_menus( NautilusActions *plugin, GList *tree, guint target, GList *files, NATokens *tokens );
 static NAObjectProfile  *get_candidate_profile( NautilusActions *plugin, NAObjectAction *action, guint target, GList *files );
 static NautilusMenuItem *create_item_from_profile( NAObjectProfile *profile, guint target, GList *files );
 static NautilusMenuItem *create_item_from_menu( NAObjectMenu *menu, GList *subitems );
@@ -95,7 +99,6 @@ static NautilusMenuItem *create_menu_item( NAObjectItem *item );
 static void              attach_submenu_to_item( NautilusMenuItem *item, GList *subitems );
 static void              weak_notify_profile( NAObjectProfile *profile, NautilusMenuItem *item );
 static void              destroy_notify_file_list( GList *list);
-static void              weak_notify_menu( NAObjectMenu *menu, NautilusMenuItem *item );
 
 static void              execute_action( NautilusMenuItem *item, NAObjectProfile *profile );
 
@@ -315,12 +318,14 @@ ipivot_consumer_items_changed( NAIPivotConsumer *instance, gpointer user_data )
 	static const gchar *thisfn = "nautilus_actions_ipivot_consumer_items_changed";
 	NautilusActions *self;
 
-	g_debug( "%s: instance=%p, user_data=%p", thisfn, ( void * ) instance, ( void * ) user_data );
 	g_return_if_fail( NAUTILUS_IS_ACTIONS( instance ));
+
 	self = NAUTILUS_ACTIONS( instance );
 
 	if( !self->private->dispose_has_run ){
 
+		g_debug( "%s: instance=%p, user_data=%p", thisfn, ( void * ) instance, ( void * ) user_data );
+
 		nautilus_menu_provider_emit_items_updated_signal( NAUTILUS_MENU_PROVIDER( self ));
 	}
 }
@@ -331,12 +336,14 @@ ipivot_consumer_create_root_menu_changed( NAIPivotConsumer *instance, gboolean e
 	static const gchar *thisfn = "nautilus_actions_ipivot_consumer_create_root_menu_changed";
 	NautilusActions *self;
 
-	g_debug( "%s: instance=%p, enabled=%s", thisfn, ( void * ) instance, enabled ? "True":"False" );
 	g_return_if_fail( NAUTILUS_IS_ACTIONS( instance ));
+
 	self = NAUTILUS_ACTIONS( instance );
 
 	if( !self->private->dispose_has_run ){
 
+		g_debug( "%s: instance=%p, enabled=%s", thisfn, ( void * ) instance, enabled ? "True":"False" );
+
 		nautilus_menu_provider_emit_items_updated_signal( NAUTILUS_MENU_PROVIDER( self ));
 	}
 }
@@ -347,12 +354,14 @@ ipivot_consumer_display_about_changed( NAIPivotConsumer *instance, gboolean enab
 	static const gchar *thisfn = "nautilus_actions_ipivot_consumer_display_about_changed";
 	NautilusActions *self;
 
-	g_debug( "%s: instance=%p, enabled=%s", thisfn, ( void * ) instance, enabled ? "True":"False" );
 	g_return_if_fail( NAUTILUS_IS_ACTIONS( instance ));
+
 	self = NAUTILUS_ACTIONS( instance );
 
 	if( !self->private->dispose_has_run ){
 
+		g_debug( "%s: instance=%p, enabled=%s", thisfn, ( void * ) instance, enabled ? "True":"False" );
+
 		nautilus_menu_provider_emit_items_updated_signal( NAUTILUS_MENU_PROVIDER( self ));
 	}
 }
@@ -363,12 +372,14 @@ ipivot_consumer_display_order_changed( NAIPivotConsumer *instance, gint order_mo
 	static const gchar *thisfn = "nautilus_actions_ipivot_consumer_display_order_changed";
 	NautilusActions *self;
 
-	g_debug( "%s: instance=%p, order_mode=%d", thisfn, ( void * ) instance, order_mode );
 	g_return_if_fail( NAUTILUS_IS_ACTIONS( instance ));
+
 	self = NAUTILUS_ACTIONS( instance );
 
 	if( !self->private->dispose_has_run ){
 
+		g_debug( "%s: instance=%p, order_mode=%d", thisfn, ( void * ) instance, order_mode );
+
 		nautilus_menu_provider_emit_items_updated_signal( NAUTILUS_MENU_PROVIDER( self ));
 	}
 }
@@ -404,6 +415,9 @@ menu_provider_get_background_items( NautilusMenuProvider *provider, GtkWidget *w
 	static const gchar *thisfn = "nautilus_actions_menu_provider_get_background_items";
 	GList *nautilus_menus_list = NULL;
 	gchar *uri;
+	GList *selected;
+
+	g_return_val_if_fail( NAUTILUS_IS_ACTIONS( provider ), NULL );
 
 	if( !NAUTILUS_ACTIONS( provider )->private->dispose_has_run ){
 
@@ -412,8 +426,11 @@ menu_provider_get_background_items( NautilusMenuProvider *provider, GtkWidget *w
 				thisfn, ( void * ) provider, ( void * ) window, ( void * ) current_folder, uri );
 		g_free( uri );
 
-		nautilus_menus_list = get_file_or_background_items(
-				NAUTILUS_ACTIONS( provider ), ITEM_TARGET_LOCATION, ( void * ) current_folder );
+		selected = na_selected_info_get_list_from_item( current_folder );
+
+		nautilus_menus_list = get_menus_items( NAUTILUS_ACTIONS( provider ), ITEM_TARGET_LOCATION, selected );
+
+		na_selected_info_free_list( selected );
 	}
 
 	return( nautilus_menus_list );
@@ -430,9 +447,7 @@ menu_provider_get_file_items( NautilusMenuProvider *provider, GtkWidget *window,
 {
 	static const gchar *thisfn = "nautilus_actions_menu_provider_get_file_items";
 	GList *nautilus_menus_list = NULL;
-
-	g_debug( "%s: provider=%p, window=%p, files=%p, count=%d",
-			thisfn, ( void * ) provider, ( void * ) window, ( void * ) files, g_list_length( files ));
+	GList *selected;
 
 	g_return_val_if_fail( NAUTILUS_IS_ACTIONS( provider ), NULL );
 
@@ -443,8 +458,14 @@ menu_provider_get_file_items( NautilusMenuProvider *provider, GtkWidget *window,
 
 	if( !NAUTILUS_ACTIONS( provider )->private->dispose_has_run ){
 
-		nautilus_menus_list = get_file_or_background_items(
-				NAUTILUS_ACTIONS( provider ), ITEM_TARGET_SELECTION, ( void * ) files );
+		g_debug( "%s: provider=%p, window=%p, files=%p, count=%d",
+				thisfn, ( void * ) provider, ( void * ) window, ( void * ) files, g_list_length( files ));
+
+		selected = na_selected_info_get_list_from_list(( GList * ) files );
+
+		nautilus_menus_list = get_menus_items( NAUTILUS_ACTIONS( provider ), ITEM_TARGET_SELECTION, selected );
+
+		na_selected_info_free_list( selected );
 	}
 
 	return( nautilus_menus_list );
@@ -462,21 +483,19 @@ menu_provider_get_toolbar_items( NautilusMenuProvider *provider, GtkWidget *wind
 	GList *nautilus_menus_list = NULL;
 	gchar *uri;
 	GList *selected;
-	GList *pivot_tree;
 
-	uri = nautilus_file_info_get_uri( current_folder );
-	g_debug( "%s: provider=%p, window=%p, current_folder=%p (%s)",
-			thisfn, ( void * ) provider, ( void * ) window, ( void * ) current_folder, uri );
-	g_free( uri );
+	g_return_val_if_fail( NAUTILUS_IS_ACTIONS( provider ), NULL );
 
 	if( !NAUTILUS_ACTIONS( provider )->private->dispose_has_run ){
 
-		pivot_tree = na_pivot_get_items( NAUTILUS_ACTIONS( provider )->private->pivot );
+		uri = nautilus_file_info_get_uri( current_folder );
+		g_debug( "%s: provider=%p, window=%p, current_folder=%p (%s)",
+				thisfn, ( void * ) provider, ( void * ) window, ( void * ) current_folder, uri );
+		g_free( uri );
 
 		selected = na_selected_info_get_list_from_item( current_folder );
 
-		nautilus_menus_list = build_nautilus_menus(
-				NAUTILUS_ACTIONS( provider ), pivot_tree, ITEM_TARGET_TOOLBAR, selected );
+		nautilus_menus_list = get_menus_items( NAUTILUS_ACTIONS( provider ), ITEM_TARGET_TOOLBAR, selected );
 
 		na_selected_info_free_list( selected );
 	}
@@ -485,52 +504,140 @@ menu_provider_get_toolbar_items( NautilusMenuProvider *provider, GtkWidget *wind
 }
 
 static GList *
-get_file_or_background_items( NautilusActions *plugin, guint target, void *selection )
+get_menus_items( NautilusActions *plugin, guint target, GList *selection )
 {
 	GList *menus_list;
-	GList *pivot_tree;
-	GList *selected;
+	NATokens *tokens;
+	GList *pivot_tree, *copy_tree;
 	gboolean root_menu;
 	gboolean add_about;
 
 	g_return_val_if_fail( NA_IS_PIVOT( plugin->private->pivot ), NULL );
 
+	tokens = na_tokens_new_from_selection( selection );
 	pivot_tree = na_pivot_get_items( plugin->private->pivot );
+	copy_tree = expand_tokens( pivot_tree, tokens );
 
-	if( target == ITEM_TARGET_LOCATION ){
-		g_return_val_if_fail( NAUTILUS_IS_FILE_INFO( selection ), NULL );
-		selected = na_selected_info_get_list_from_item( NAUTILUS_FILE_INFO( selection ));
+	menus_list = build_nautilus_menus( plugin, copy_tree, target, selection, tokens );
 
-	} else {
-		g_return_val_if_fail( target == ITEM_TARGET_SELECTION, NULL );
-		selected = na_selected_info_get_list_from_list(( GList * ) selection );
-	}
+	na_object_unref_items( copy_tree );
+	g_object_unref( tokens );
 
-	menus_list = build_nautilus_menus( plugin, pivot_tree, target, selected );
-	/*g_debug( "%s: menus has %d level zero items", thisfn, g_list_length( menus_list ));*/
+	if( target != ITEM_TARGET_TOOLBAR ){
 
-	na_selected_info_free_list( selected );
+		root_menu = na_iprefs_read_bool( NA_IPREFS( plugin->private->pivot ), IPREFS_CREATE_ROOT_MENU, FALSE );
+		if( root_menu ){
+			menus_list = create_root_menu( plugin, menus_list );
+		}
 
-	root_menu = na_iprefs_read_bool( NA_IPREFS( plugin->private->pivot ), IPREFS_CREATE_ROOT_MENU, FALSE );
-	if( root_menu ){
-		menus_list = create_root_menu( plugin, menus_list );
-	}
-
-	add_about = na_iprefs_read_bool( NA_IPREFS( plugin->private->pivot ), IPREFS_ADD_ABOUT_ITEM, TRUE );
-	/*g_debug( "%s: add_about=%s", thisfn, add_about ? "True":"False" );*/
-	if( add_about ){
-		menus_list = add_about_item( plugin, menus_list );
+		add_about = na_iprefs_read_bool( NA_IPREFS( plugin->private->pivot ), IPREFS_ADD_ABOUT_ITEM, TRUE );
+		if( add_about ){
+			menus_list = add_about_item( plugin, menus_list );
+		}
 	}
 
 	return( menus_list );
 }
 
 /*
- * when building a menu for the toolbar, do not use menus hierarchy
- * files is a GList of NASelectedInfo items
+ * create a copy of the tree where all fields which may embed parameters
+ * have been expanded
  */
 static GList *
-build_nautilus_menus( NautilusActions *plugin, GList *tree, guint target, GList *files )
+expand_tokens( GList *pivot_tree, NATokens *tokens )
+{
+	GList *tree, *it;
+
+	tree = NULL;
+
+	for( it = pivot_tree ; it ; it = it->next ){
+		NAObjectItem *item = NA_OBJECT_ITEM( na_object_duplicate( it->data ));
+		tree = g_list_prepend( tree, expand_tokens_item( item, tokens ));
+	}
+
+	return( g_list_reverse( tree ));
+}
+
+static NAObjectItem *
+expand_tokens_item( NAObjectItem *item, NATokens *tokens )
+{
+	gchar *old, *new;
+	GSList *subitems_slist, *its, *new_slist;
+	GList *subitems, *it, *new_list;
+
+	old = na_object_get_label( item );
+	new = na_tokens_parse_parameters( tokens, old, TRUE );
+	na_object_set_label( item, new );
+	g_free( old );
+	g_free( new );
+
+	old = na_object_get_tooltip( item );
+	new = na_tokens_parse_parameters( tokens, old, TRUE );
+	na_object_set_tooltip( item, new );
+	g_free( old );
+	g_free( new );
+
+	old = na_object_get_icon( item );
+	new = na_tokens_parse_parameters( tokens, old, TRUE );
+	na_object_set_icon( item, new );
+	g_free( old );
+	g_free( new );
+
+	if( NA_IS_OBJECT_ACTION( item )){
+		old = na_object_get_toolbar_label( item );
+		new = na_tokens_parse_parameters( tokens, old, TRUE );
+		na_object_set_toolbar_label( item, new );
+		g_free( old );
+		g_free( new );
+	}
+
+	subitems_slist = na_object_get_items_slist( item );
+	new_slist = NULL;
+	for( its = subitems_slist ; its ; its = its->next ){
+		old = ( gchar * ) its->data;
+		new = na_tokens_parse_parameters( tokens, old, FALSE );
+		new_slist = g_slist_prepend( new_slist, new );
+	}
+	na_object_set_items_slist( item, new_slist );
+	na_core_utils_slist_free( subitems_slist );
+	na_core_utils_slist_free( new_slist );
+
+	subitems = na_object_get_items( item );
+	new_list = NULL;
+	for( it = subitems ; it ; it = it->next ){
+		if( NA_IS_OBJECT_MENU( item )){
+			expand_tokens_item( NA_OBJECT_ITEM( it->data ), tokens );
+
+		} else {
+			old = na_object_get_path( it->data );
+			new = na_tokens_parse_parameters( tokens, old, FALSE );
+			na_object_set_path( it->data, new );
+			g_free( old );
+			g_free( new );
+
+			old = na_object_get_parameters( it->data );
+			new = na_tokens_parse_parameters( tokens, old, FALSE );
+			na_object_set_parameters( it->data, new );
+			g_free( old );
+			g_free( new );
+		}
+	}
+
+	return( item );
+}
+
+/*
+ * when building a menu for the toolbar, do not use menus hierarchy
+ * files is a GList of NASelectedInfo items
+ *
+ * TODO: menus, actions and profiles may embed parameters in their data.
+ * This mainly means that the objects have to be re-parsed for each new
+ * selection (e.g. because a label may change if it depends of the current
+ * selection). Thus, all the hierarchy must be recursively re-parsed, and
+ * should be re-checked for validity !
+ */
+static GList *
+build_nautilus_menus( NautilusActions *plugin, GList *tree, guint target, GList *files, NATokens *tokens )
 {
 	static const gchar *thisfn = "nautilus_actions_build_nautilus_menus";
 	GList *menus_list = NULL;
@@ -538,7 +645,6 @@ build_nautilus_menus( NautilusActions *plugin, GList *tree, guint target, GList
 	GList *it;
 	NAObjectProfile *profile;
 	NautilusMenuItem *item;
-	gchar *action_label;
 
 	g_debug( "%s: plugin=%p, tree=%p, target=%d, files=%p (count=%d)",
 			thisfn, ( void * ) plugin, ( void * ) tree, target,
@@ -562,15 +668,15 @@ build_nautilus_menus( NautilusActions *plugin, GList *tree, guint target, GList
 		}
 #endif
 
-		/*if( !na_icontext_is_candidate( it->data, target, files )){
+		if( !na_icontext_is_candidate( NA_ICONTEXT( it->data ), target, files )){
 			continue;
-		}*/
+		}
 
 		/* recursively build sub-menus
 		 */
 		if( NA_IS_OBJECT_MENU( it->data )){
 			subitems = na_object_get_items( it->data );
-			submenu = build_nautilus_menus( plugin, subitems, target, files );
+			submenu = build_nautilus_menus( plugin, subitems, target, files, tokens );
 			/*g_debug( "%s: submenu has %d items", thisfn, g_list_length( submenu ));*/
 
 			if( submenu ){
@@ -586,24 +692,12 @@ build_nautilus_menus( NautilusActions *plugin, GList *tree, guint target, GList
 		}
 
 		g_return_val_if_fail( NA_IS_OBJECT_ACTION( it->data ), NULL );
-		action_label = na_object_get_label( it->data );
 
-		/* to be removed when NAObjectAction will implement NAIContext interface
-		 */
-		if( !na_object_action_is_candidate( it->data, target, files )){
-			g_debug( "%s: action %s is not candidate", thisfn, action_label );
-			g_free( action_label );
-			continue;
-		}
-
-		g_debug( "%s: action %s is candidate", thisfn, action_label );
 		profile = get_candidate_profile( plugin, NA_OBJECT_ACTION( it->data ), target, files );
 		if( profile ){
 			item = create_item_from_profile( profile, target, files );
 			menus_list = g_list_append( menus_list, item );
 		}
-
-		g_free( action_label );
 	}
 
 	return( menus_list );
@@ -702,7 +796,6 @@ create_item_from_menu( NAObjectMenu *menu, GList *subitems )
 	NautilusMenuItem *item;
 
 	item = create_menu_item( NA_OBJECT_ITEM( menu ));
-	g_object_weak_ref( G_OBJECT( item ), ( GWeakNotify ) weak_notify_menu, menu );
 
 	attach_submenu_to_item( item, subitems );
 	nautilus_menu_item_list_free( subitems );
@@ -711,14 +804,6 @@ create_item_from_menu( NAObjectMenu *menu, GList *subitems )
 	return( item );
 }
 
-static void
-weak_notify_menu( NAObjectMenu *menu, NautilusMenuItem *item )
-{
-	g_debug( "nautilus_actions_weak_notify_menu: menu=%p (ref_count=%d)",
-			( void * ) menu, G_OBJECT( menu )->ref_count );
-	/*g_object_unref( menu );*/
-}
-
 static NautilusMenuItem *
 create_menu_item( NAObjectItem *item )
 {
